#include "mesh_ug_io.h"
#include "libmeshb7.h"

INT_ mesh_ug_io_write_func (
  char File_Name[],
  INT_ NDim,
  INT_ Number_of_Nodes,
  INT_ Number_of_U_Scalars,
  INT_ Number_of_U_Vectors,
  INT_ Number_of_U_Metrics,
  INT_ Number_of_U_SymMetrics,
  DOUBLE_1D * U_Scalars,
  DOUBLE_1D * U_Vectors,
  DOUBLE_1D * U_Metrics,
  DOUBLE_1D * U_SymMetrics)
{

/*
 * Write function data to a SOL solution file.
 * 
 * $Id: mesh_ug_io_write_func.c,v 1.8 2019/11/12 13:23:11 marcum Exp $
 */

  DOUBLE_1D *U_Scalar = NULL;
  DOUBLE_1D *U_Vector = NULL;
  DOUBLE_1D *U_Metric = NULL;
  DOUBLE_1D *U_SymMetric = NULL;

  int TypTab[ GmfMaxTyp ];
  int FilVer;

  LONG_LONG_int Sol_File_Index;

  INT_ i, j, n, Index, Number_of_Functions;
  INT_ U_Scalar_Location = 0;
  INT_ U_Vector_Location = 0;
  INT_ U_Metric_Location = 0;
  INT_ U_SymMetric_Location = 0;

  double bufDbl[9];
  double Size;
  double SizeLimit = 1.95;

  // set number of functions and type

  if (U_Scalars == NULL) Number_of_U_Scalars = 0;
  if (U_Vectors == NULL) Number_of_U_Vectors = 0;
  if (U_Metrics == NULL) Number_of_U_Metrics = 0;
  if (U_SymMetrics == NULL) Number_of_U_SymMetrics = 0;

  Number_of_Functions = Number_of_U_Scalars + Number_of_U_Vectors
                      + Number_of_U_Metrics + Number_of_U_SymMetrics;

  // check input data

  if (Number_of_Functions == 0 ) {
    ug_error_message ("*** ERROR : no function fields specified for SOL file ***");
    return 1;
  }

  if (Number_of_Nodes == 0 ) {
    ug_error_message ("*** ERROR : no vertices/nodes specified for SOL file ***");
    return 1;
  }

  // set function type table

  j = 0;

  for (i = j; i < j + (int) (Number_of_U_Scalars); i++) {
    TypTab[i] = GmfSca;  
  }

  j = j + (int) (Number_of_U_Scalars);

  for (i = j; i < j + (int) (Number_of_U_Vectors); i++) {
    TypTab[i] = GmfVec;  
  }

  j = j + (int) (Number_of_U_Vectors);

  for (i = j; i < j + (int) (Number_of_U_Metrics); i++) {
    TypTab[i] = GmfMat;  
  }

  j = j + (int) (Number_of_U_Metrics);
  
  for (i = j; i < j + (int) (Number_of_U_SymMetrics); i++) {
    TypTab[i] = GmfSymMat;  
  }

  j = j + (int) (Number_of_U_SymMetrics);

  // set file version

  Size = ((double) ((sizeof (DOUBLE_1D)) * Number_of_U_Scalars * Number_of_Nodes
                  + (sizeof (DOUBLE_3D)) * Number_of_U_Vectors * Number_of_Nodes
                  + (sizeof (DOUBLE_9D)) * Number_of_U_Metrics * Number_of_Nodes
                  + (sizeof (DOUBLE_6D)) * Number_of_U_SymMetrics * Number_of_Nodes))
       / ((double) (1024 * 1024 * 1024));

  FilVer = (sizeof (INT_) == 8) ? 4: 
           (Size > SizeLimit) ? 3: 2;

  // write function data

  Sol_File_Index = GmfOpenMesh(File_Name, GmfWrite, FilVer, (int) NDim); 
  
  if ( Sol_File_Index <= 0 ) {
    ug_error_message ("*** ERROR : error opening SOL file ***");
    return 1;
  }
  
  GmfSetKwd(Sol_File_Index, GmfSolAtVertices, Number_of_Nodes, Number_of_Functions, TypTab);

  for (i = 0; i < Number_of_U_Scalars; i++) {

    U_Scalar = &U_Scalars[U_Scalar_Location];

    for (Index = 1; Index <= Number_of_Nodes; Index++) {

      bufDbl[0] = U_Scalar[Index];

      GmfSetLin(Sol_File_Index, GmfSolAtVertices, bufDbl);
    }

    U_Scalar_Location = U_Scalar_Location + Number_of_Nodes + 1;
  }

  n = NDim;

  for (i = 0; i < Number_of_U_Vectors; i++) {

    U_Vector = &U_Vectors[U_Vector_Location];

    for (Index = 1; Index <= Number_of_Nodes; Index++) {

      for (j = 0; j < n; j++) {
        bufDbl[j] = U_Vector[j+Index*n];
      }

      GmfSetLin(Sol_File_Index, GmfSolAtVertices, bufDbl);
    }

    U_Vector_Location = U_Vector_Location + Number_of_Nodes + 1;
  }

  n = NDim*NDim;

  for (i = 0; i < Number_of_U_Metrics; i++) {

    U_Metric = &U_Metrics[U_Metric_Location];

    for (Index = 1; Index <= Number_of_Nodes; Index++) {

      for (j = 0; j < n; j++) {
        bufDbl[j] = U_Metric[j+Index*n];
      }

      GmfSetLin(Sol_File_Index, GmfSolAtVertices, bufDbl);
    }

    U_Metric_Location = U_Metric_Location + Number_of_Nodes + 1;
  }

  n = 3*(NDim-1);

  for (i = 0; i < Number_of_U_SymMetrics; i++) {

    U_SymMetric = &U_SymMetrics[U_SymMetric_Location];

    for (Index = 1; Index <= Number_of_Nodes; Index++) {

      for (j = 0; j < n; j++) {
        bufDbl[j] = U_SymMetric[j+Index*n];
      }

      // WARNING: Gamma convention switch 2 and 3

      if (NDim == 3)
      {
        bufDbl[2] = U_SymMetric[3+Index*n];
        bufDbl[3] = U_SymMetric[2+Index*n];
      }

      GmfSetLin(Sol_File_Index, GmfSolAtVertices, bufDbl);
    }

    U_SymMetric_Location = U_SymMetric_Location + Number_of_Nodes + 1;
  }

  GmfCloseMesh(Sol_File_Index);

  return 0;
}
