#include "UG2_LIB.h"

/*
 * UG2 LIB : Unstructured Grid - General Purpose Routine Library
 * 2D Version : $Id: ug2_bv.c,v 1.6 2020/02/15 05:24:54 marcum Exp $
 * Copyright 1994-2020, David L. Marcum
 */

int ug2_conv_3d_ax (
  INT_ ntria_2d,
  INT_ nquad_2d,
  INT_ nnode_2d,
  INT_4D *iniq_2d,
  INT_3D *init_2d,
  DOUBLE_2D *x_2d,
  INT_ *nbface,
  INT_ *nquad,
  INT_ *nelem,
  INT_ *nelemc5,
  INT_ *nelemc6,
  INT_ *nelemc8,
  INT_ *nnode,
  INT_3D **inibf,
  INT_4D **iniq,
  INT_4D **iniel,
  INT_5D **inielc5,
  INT_6D **inielc6,
  INT_8D **inielc8,
  DOUBLE_3D **x)
{
  // Convert 2D planar mesh to a 3D axisymmetric mesh.

    int i__1, i__2;

    /* Local variables */
    double r__, x1, x2, x3;
    int id, id1, id2;
    double 0.0, 1.0, ang;
    int ncl;
    double 45.0, ang0, pid4;
    int ntet;
    double dang0;
    int npip0, npip1, npip2, npip3, ielem, idmax, ipoin, jpoin, 
      ipoin1, ipoin2, ipoin3, ipoin4, jpoin1, jpoin2, jpoin3, jpoin4, 
      jpoin5, jpoin6, ibface, iplane, ipoinm, nstria, nnodep, ntpoin, 
      nplane1, nplane2;

    /* Parameter adjustments */
    ipibf -= 5;
    ipiel -= 4;
    x -= 3;
    --npip;

  nplane1 = nplane - 1;
  nplane2 = nplane - 2;
  pid4 = atan(1.0);
  dang0 = dang * atan(1.0) / 45.0;
  ang0 = atan(1.0) + atan(1.0) + dang0 * (double) (nplane - 1);

  jnode = 0;
  ncl = 0;

  for (inode = 1; inode <= nnode_2d; inode++) {

    if (x_2d[inode][1] == 0.0) {
      ncl++;
      npip[inode] = 0;
    }
    else {
      jnode++;
      npip[inode] = nnode_2d + jnode - inode;
    }
  }

  nnodep = nnode_2d - ncl;
  *nnode = nnode_2d + (nplane-1) * nnodep;

  nstria = *nelem << 1;
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 1];
      ipoin2 = ipibf[(ibface << 2) + 2];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if ((npip1 == 0 || npip2 == 0) && (npip1 > 0 || npip2 > 0)) {
    nstria += nplane1;
      }
  }
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 2];
      ipoin2 = ipibf[(ibface << 2) + 1];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if (npip1 > 0 && npip2 > 0) {
    nstria += nplane1 << 1;
      }
  }
  ntet = 0;
  i__1 = *nelem;
  for (ielem = 1; ielem <= i__1; ++ielem) {
      ipoin1 = ipiel[ielem * 3 + 1];
      ipoin2 = ipiel[ielem * 3 + 2];
      ipoin3 = ipiel[ielem * 3 + 3];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      npip3 = npip[ipoin3];
/* Computing MIN */
      i__2 = MIN(npip1,npip2);
      npip0 = MIN(i__2,npip3);
      if (npip0 > 0) {
    ntet += nplane1 * 3;
      }
      if ((npip1 == 0 && npip2 == 0) || (npip2 == 0 && npip3 == 0) || (npip3 
        == 0 && npip1 == 0)) {
    ntet += nplane1;
      }
      if ((npip1 == 0 && npip2 > 0 && npip3 > 0) || (npip2 == 0 && npip3 > 
        0 && npip1 > 0) || (npip3 == 0 && npip1 > 0 && npip2 > 0)) {
    ntet += nplane1 << 1;
      }
  }
  fprintf (grid_file, "%i %i %i\n", ntpoin, nstria,ntet);
  i__1 = nplane;
  for (iplane = 1; iplane <= i__1; ++iplane) {
      for (ipoin = 1; ipoin <= nnode_2d; ++ipoin) {
    if (iplane == 1 || npip[ipoin] > 0) {
        x1 = x[(ipoin << 1) + 1];
        fprintf (grid_file, "%25.16g\n", x1);
    }
      }
  }
  i__1 = nplane;
  for (iplane = 1; iplane <= i__1; ++iplane) {
      ang = ang0 - dang0 * (double) (iplane - 1);
      for (ipoin = 1; ipoin <= nnode_2d; ++ipoin) {
    if (iplane == 1 || npip[ipoin] > 0) {
        r__ = x[(ipoin << 1) + 2];
        x2 = r__ * sin(ang);
        fprintf (grid_file, "%25.16g\n", x2);
    }
      }
  }
  i__1 = nplane;
  for (iplane = 1; iplane <= i__1; ++iplane) {
      ang = ang0 - dang0 * (double) (iplane - 1);
      for (ipoin = 1; ipoin <= nnode_2d; ++ipoin) {
    if (iplane == 1 || npip[ipoin] > 0) {
        r__ = x[(ipoin << 1) + 2];
        x3 = r__ * cos(ang);
        fprintf (grid_file, "%25.16g\n", x3);
    }
      }
  }
  i__1 = *nelem;
  for (ielem = 1; ielem <= i__1; ++ielem) {
      ipoin1 = ipiel[ielem * 3 + 1];
      ipoin2 = ipiel[ielem * 3 + 2];
      ipoin3 = ipiel[ielem * 3 + 3];
            fprintf (grid_file, "%i %i %i\n", ipoin2, ipoin1, ipoin3);
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      npip3 = npip[ipoin3];
      ipoin1 += npip1;
      ipoin2 += npip2;
      ipoin3 += npip3;
      if (npip1 > 0) {
    ipoin1 += nplane2 * nnodep;
      }
      if (npip2 > 0) {
    ipoin2 += nplane2 * nnodep;
      }
      if (npip3 > 0) {
    ipoin3 += nplane2 * nnodep;
      }
            fprintf (grid_file, "%i %i %i\n", ipoin1, ipoin2, ipoin3);
  }
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 1];
      ipoin2 = ipibf[(ibface << 2) + 2];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if ((npip1 == 0 || npip2 == 0) && (npip1 > 0 || npip2 > 0)) {
    if (npip1 == 0) {
        ipoin3 = ipoin2 + npip2;
                    fprintf (grid_file, "%i %i %i\n", ipoin3, ipoin1, ipoin2);
        ipoin2 = ipoin3;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      ipoin3 = ipoin2 + nnodep;
                        fprintf (grid_file, "%i %i %i\n", ipoin3, ipoin1, ipoin2);
      ipoin2 = ipoin3;
        }
    } else {
        ipoin3 = ipoin1 + npip1;
                    fprintf (grid_file, "%i %i %i\n", ipoin3, ipoin1, ipoin2);
        ipoin1 = ipoin3;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      ipoin3 = ipoin1 + nnodep;
                        fprintf (grid_file, "%i %i %i\n", ipoin3, ipoin1, ipoin2);
      ipoin1 = ipoin3;
        }
    }
      }
  }
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 2];
      ipoin2 = ipibf[(ibface << 2) + 1];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if (npip1 > 0 && npip2 > 0) {
    if (ipoin1 < ipoin2) {
        ipoin3 = ipoin2 + npip2;
        ipoin4 = ipoin1 + npip1;
                    fprintf (grid_file, "%i %i %i\n",ipoin2, ipoin1, ipoin4);
                    fprintf (grid_file, "%i %i %i\n",ipoin3, ipoin2, ipoin4);
        ipoin1 = ipoin4;
        ipoin2 = ipoin3;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      ipoin3 = ipoin2 + nnodep;
      ipoin4 = ipoin1 + nnodep;
                        fprintf (grid_file, "%i %i %i\n",ipoin2, ipoin1, ipoin4);
                        fprintf (grid_file, "%i %i %i\n",ipoin3, ipoin2, ipoin4);
      ipoin1 = ipoin4;
      ipoin2 = ipoin3;
        }
    } else {
        ipoin3 = ipoin2 + npip2;
        ipoin4 = ipoin1 + npip1;
                    fprintf (grid_file, "%i %i %i\n",ipoin2, ipoin1, ipoin3);
                    fprintf (grid_file, "%i %i %i\n",ipoin3, ipoin1, ipoin4);
        ipoin1 = ipoin4;
        ipoin2 = ipoin3;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      ipoin3 = ipoin2 + nnodep;
      ipoin4 = ipoin1 + nnodep;
                        fprintf (grid_file, "%i %i %i\n",ipoin2, ipoin1, ipoin3);
                        fprintf (grid_file, "%i %i %i\n",ipoin3, ipoin1, ipoin4);
      ipoin1 = ipoin4;
      ipoin2 = ipoin3;
        }
    }
      }
  }
  idmax = ipibf[8];
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      id = ipibf[(ibface << 2) + 4];
      idmax = MAX(id,idmax);
  }
  id1 = idmax + 1;
  id2 = idmax + 2;
  i__1 = *nelem;
  for (ielem = 1; ielem <= i__1; ++ielem) {
      fprintf(grid_file, "%i\n", id1);
      fprintf(grid_file, "%i\n", id2);
  }
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 1];
      ipoin2 = ipibf[(ibface << 2) + 2];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if ((npip1 == 0 || npip2 == 0) && (npip1 > 0 || npip2 > 0)) {
    id = ipibf[(ibface << 2) + 4];
    i__2 = nplane1;
    for (iplane = 1; iplane <= i__2; ++iplane) {
        fprintf(grid_file, "%i\n", id);
    }
      }
  }
  for (ibface = 1; ibface <= ntria_2d; ++ibface) {
      ipoin1 = ipibf[(ibface << 2) + 2];
      ipoin2 = ipibf[(ibface << 2) + 1];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      if (npip1 > 0 && npip2 > 0) {
    id = ipibf[(ibface << 2) + 4];
    i__2 = nplane1;
    for (iplane = 1; iplane <= i__2; ++iplane) {
              fprintf(grid_file, "%i\n", id);
              fprintf(grid_file, "%i\n", id);
    }
      }
  }
  i__1 = *nelem;
  for (ielem = 1; ielem <= i__1; ++ielem) {
      ipoin1 = ipiel[ielem * 3 + 1];
      ipoin2 = ipiel[ielem * 3 + 2];
      ipoin3 = ipiel[ielem * 3 + 3];
      npip1 = npip[ipoin1];
      npip2 = npip[ipoin2];
      npip3 = npip[ipoin3];
/* Computing MIN */
      i__2 = MIN(npip1,npip2);
      npip0 = MIN(i__2,npip3);
      if (npip0 > 0) {
/* Computing MIN */
    i__2 = MIN(ipoin1,ipoin2);
    ipoinm = MIN(i__2,ipoin3);
    if (ipoin1 == ipoinm) {
        jpoin1 = ipoin1;
        jpoin2 = ipoin2;
        jpoin3 = ipoin3;
    } else {
        if (ipoin2 == ipoinm) {
      jpoin1 = ipoin2;
      jpoin2 = ipoin3;
      jpoin3 = ipoin1;
        } else {
      jpoin1 = ipoin3;
      jpoin2 = ipoin1;
      jpoin3 = ipoin2;
        }
    }
    if (jpoin2 < jpoin3) {
        npip1 = npip[jpoin1];
        npip2 = npip[jpoin2];
        npip3 = npip[jpoin3];
        jpoin4 = jpoin1 + npip1;
        jpoin5 = jpoin2 + npip2;
        jpoin6 = jpoin3 + npip3;
        fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
        fprintf(grid_file, "%i %i %i %i\n", jpoin2, jpoin3, jpoin4, jpoin5);
        fprintf(grid_file, "%i %i %i %i\n", jpoin3, jpoin4, jpoin5, jpoin6);
        jpoin1 = jpoin4;
        jpoin2 = jpoin5;
        jpoin3 = jpoin6;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      jpoin4 = jpoin1 + nnodep;
      jpoin5 = jpoin2 + nnodep;
      jpoin6 = jpoin3 + nnodep;
            fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
            fprintf(grid_file, "%i %i %i %i\n", jpoin2, jpoin3, jpoin4, jpoin5);
            fprintf(grid_file, "%i %i %i %i\n", jpoin3, jpoin4, jpoin5, jpoin6);
      jpoin1 = jpoin4;
      jpoin2 = jpoin5;
      jpoin3 = jpoin6;
        }
    } else {
        npip1 = npip[jpoin1];
        npip2 = npip[jpoin2];
        npip3 = npip[jpoin3];
        jpoin4 = jpoin1 + npip1;
        jpoin5 = jpoin2 + npip2;
        jpoin6 = jpoin3 + npip3;
        fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
        fprintf(grid_file, "%i %i %i %i\n", jpoin3, jpoin4, jpoin2, jpoin6);
        fprintf(grid_file, "%i %i %i %i\n", jpoin2, jpoin6, jpoin4, jpoin5);
        jpoin1 = jpoin4;
        jpoin2 = jpoin5;
        jpoin3 = jpoin6;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      jpoin4 = jpoin1 + nnodep;
      jpoin5 = jpoin2 + nnodep;
      jpoin6 = jpoin3 + nnodep;
            fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
            fprintf(grid_file, "%i %i %i %i\n", jpoin3, jpoin4, jpoin2, jpoin6);
            fprintf(grid_file, "%i %i %i %i\n", jpoin2, jpoin6, jpoin4, jpoin5);
      jpoin1 = jpoin4;
      jpoin2 = jpoin5;
      jpoin3 = jpoin6;
        }
    }
      } else {
    if (npip1 == 0 && npip2 == 0) {
        jpoin1 = ipoin1;
        jpoin2 = ipoin2;
        jpoin3 = ipoin3;
    } else {
        if (npip2 == 0 && npip3 == 0) {
      jpoin1 = ipoin2;
      jpoin2 = ipoin3;
      jpoin3 = ipoin1;
      npip3 = npip1;
        } else {
      if (npip3 == 0 && npip1 == 0) {
          jpoin1 = ipoin3;
          jpoin2 = ipoin1;
          jpoin3 = ipoin2;
          npip3 = npip2;
      } else {
          jpoin1 = 0;
      }
        }
    }
    if (jpoin1 > 0) {
        jpoin4 = jpoin3 + npip3;
        fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
        jpoin3 = jpoin4;
        i__2 = nplane1;
        for (iplane = 2; iplane <= i__2; ++iplane) {
      jpoin4 = jpoin3 + nnodep;
            fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
      jpoin3 = jpoin4;
        }
    } else {
        if (npip1 == 0 && npip2 > 0 && npip3 > 0) {
      jpoin1 = ipoin1;
      jpoin2 = ipoin2;
      jpoin3 = ipoin3;
        } else {
      if (npip2 == 0 && npip1 > 0 && npip3 > 0) {
          jpoin1 = ipoin2;
          jpoin2 = ipoin3;
          jpoin3 = ipoin1;
          npip2 = npip3;
          npip3 = npip1;
      } else {
          if (npip3 == 0 && npip1 > 0 && npip2 > 0) {
        jpoin1 = ipoin3;
        jpoin2 = ipoin1;
        jpoin3 = ipoin2;
        npip3 = npip2;
        npip2 = npip1;
          }
      }
        }
        if (jpoin1 > 0) {
      if (jpoin2 < jpoin3) {
          jpoin4 = jpoin2 + npip2;
          jpoin5 = jpoin3 + npip3;
                fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
                fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin4, jpoin3, jpoin5);
          jpoin2 = jpoin4;
          jpoin3 = jpoin5;
          i__2 = nplane1;
          for (iplane = 2; iplane <= i__2; ++iplane) {
        jpoin4 = jpoin2 + nnodep;
        jpoin5 = jpoin3 + nnodep;
                    fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin4);
                    fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin4, jpoin3, jpoin5);
        jpoin2 = jpoin4;
        jpoin3 = jpoin5;
          }
      } else {
          jpoin4 = jpoin2 + npip2;
          jpoin5 = jpoin3 + npip3;
                fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin5);
                fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin5, jpoin4);
          jpoin2 = jpoin4;
          jpoin3 = jpoin5;
          i__2 = nplane1;
          for (iplane = 2; iplane <= i__2; ++iplane) {
        jpoin4 = jpoin2 + nnodep;
        jpoin5 = jpoin3 + nnodep;
                    fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin3, jpoin5);
                    fprintf(grid_file, "%i %i %i %i\n", jpoin1, jpoin2, jpoin5, jpoin4);
        jpoin2 = jpoin4;
        jpoin3 = jpoin5;
          }
      }
        }
    }
      }
  }
  *merror = 0;
    }
    return 0;
} /* g2w011_ */

