#include "UG2_LIB.h"

void ug2_reorderb
 (INT_ nnode,
  INT_ *nnodeb,
  INT_ nbedge,
  INT_ nelem,
  INT_ nquad,
  INT_2D * inibe,
  INT_3D * iniel,
  INT_4D * iniq,
  INT_1D * jnin,
  INT_1D * mn,
  DOUBLE_2D * x)

{

/*
 * Re-orders nodes so that all boundary nodes are first in the ordering.
 *
 * UG2 LIB : Unstructured Grid - General Purpose Routine Library
 * 2D Version : $Id: ug2_reorderb.c,v 1.8 2020/02/15 05:24:54 marcum Exp $
 * Copyright 1994-2020, David L. Marcum
 */

  INT_ found, ibedge, ielem, inode, inode0, inode1, inode2, inode3, inode4,
       iquad, jnode;

  double x1, x2;

  for (inode = 1; inode <= nnode; ++inode)
  {
    mn[inode] = 1;
  }

  for (ibedge = 1; ibedge <= nbedge; ++ibedge)
  {
    inode1 = inibe[ibedge][0];
    inode2 = inibe[ibedge][1];

    mn[inode1] = 0;
    mn[inode2] = 0;
  }

  inode0 = 0;

  *nnodeb = 0;

  for (inode = 1; inode <= nnode; ++inode)
  {
    if (mn[inode] == 0)
      ++(*nnodeb);
    else if (inode0 == 0)
      inode0 = inode;
  }

  if (inode0 == 0)
    return;

  for (inode = inode0; inode <= nnode; ++inode)
  {
    jnin[inode] = inode;
  }

  found = 1;

  inode2 = nnode;

  inode = inode0;

  do
  {
    if (mn[inode])
    {
      found = 0;

      if (inode2 > inode)
      {
        jnode = inode2;

        do
        {
          if (mn[jnode] == 0)
            found = 1;

          --jnode;
        }
        while (jnode > inode && found == 0);

        if (found == 1)
        {
          ++jnode;

          jnin[inode] = jnode;
          jnin[jnode] = inode;

          inode2 = jnode - 1;
        }
      }
    }

    ++inode;
  }
  while (inode <= nnode && found == 1);

  for (inode = 1; inode < inode0; ++inode)
  {
    jnin[inode] = inode;
  }

  for (ibedge = 1; ibedge <= nbedge; ++ibedge)
  {
    inode1 = inibe[ibedge][0];
    inode2 = inibe[ibedge][1];

    inibe[ibedge][0] = jnin[inode1];
    inibe[ibedge][1] = jnin[inode2];
  }

  for (ielem = 1; ielem <= nelem; ++ielem)
  {
    inode1 = iniel[ielem][0];
    inode2 = iniel[ielem][1];
    inode3 = iniel[ielem][2];

    iniel[ielem][0] = jnin[inode1];
    iniel[ielem][1] = jnin[inode2];
    iniel[ielem][2] = jnin[inode3];
  }

  for (iquad = 1; iquad <= nquad; ++iquad)
  {
    inode1 = iniq[iquad][0];
    inode2 = iniq[iquad][1];
    inode3 = iniq[iquad][2];
    inode4 = iniq[iquad][3];

    iniq[iquad][0] = jnin[inode1];
    iniq[iquad][1] = jnin[inode2];
    iniq[iquad][2] = jnin[inode3];
    iniq[iquad][3] = jnin[inode4];
  }

  for (inode = 1; inode <= nnode; ++inode)
  {
    jnode = jnin[inode];

    if (jnode > inode)
    {
      x1 = x[inode][0];
      x2 = x[inode][1];

      x[inode][0] = x[jnode][0];
      x[inode][1] = x[jnode][1];

      x[jnode][0] = x1;
      x[jnode][1] = x2;
    }
  }

  return;

}
