#include "UG3_LIB.h"

INT_ ug3_read_bgdata_file (char Case_Name[],
                           INT_ mmsg,
                           INT_ *mmetbg,
                           INT_ *nelbg,
                           INT_ *nnbg,
                           INT_4D ** ielielbg,
                           INT_4D ** inielbg,
                           DOUBLE_1D ** dfbg,
                           DOUBLE_6D ** metbg,
                           DOUBLE_3D ** xbg)
{

/*
 * Read BGDATA background mesh data file.
 * 
 * SD3 LIB : Sub-Domain Utility Library
 * 3D Version : $Id: ug3_read_bgdata_file.c,v 1.12 2022/11/21 00:25:14 marcum Exp $
 * Copyright 1994-2021, David L. Marcum
 */

  FILE *Data_File = NULL;

  CHAR_UG_MAX drive, dname, ext, fname, File_Name, Text;
  CHAR_51 Name_Text;

  size_t ir = 0;
  INT_ ierr = 0;

  *nelbg = 0;
  *nnbg = 0;
  *ielielbg = NULL;
  *inielbg = NULL;
  *dfbg = NULL;
  *metbg = NULL;
  *xbg = NULL;

  ug_set_byte_order (1);

  strcpy (File_Name, Case_Name);

  if (sizeof (INT_) == sizeof (int))
    strcat (File_Name, ".b8.bgdata");
  else
    strcat (File_Name, ".b8l.bgdata");

  ug_splitpath (File_Name, drive, dname, fname, ext);

  strcat (fname, ext);
  strcpy (Name_Text, "");
  strncat (Name_Text, fname, 50);

  if (mmsg >= 1)
  {
    ug_message ("");
    ug_message ("UG3      : READ BACKGROUND MESH DATA FILE");
    ug_message ("");
    snprintf (Text, sizeof(Text), "UG3      : BG Data File Name = %s", Name_Text);
    ug_message (Text);

    ug_cpu_message ("");
  }

  Data_File = ug_fopen (File_Name, "r");

  if (Data_File == NULL)
  {
    ug_error_message ("*** ERROR 363 : error opening BG data file ***");
    return (363);
  }

  ir = ir + ug_fread (mmetbg, sizeof (INT_), 1, Data_File);
  ir = ir - 1;
  ir = ir + ug_fread (nelbg, sizeof (INT_), 1, Data_File);
  ir = ir - 1;
  ir = ir + ug_fread (nnbg, sizeof (INT_), 1, Data_File);
  ir = ir - 1;

  if (ir != 0)
  {
    ug_fclose (Data_File);
    ug_error_message ("*** ERROR 364 : error reading BGDATA file ***");
    return (364);
  }

  *ielielbg = (INT_4D *) ug_malloc (&ierr, ((*nelbg)+1) * sizeof (INT_4D));
  *inielbg = (INT_4D *) ug_malloc (&ierr, ((*nelbg)+1) * sizeof (INT_4D));

  *dfbg = (DOUBLE_1D *) ug_malloc (&ierr, ((*nnbg)+1) * sizeof (DOUBLE_1D));
  if (*mmetbg) *metbg = (DOUBLE_6D *) ug_malloc (&ierr, ((*nnbg)+1) * sizeof (DOUBLE_6D));
  *xbg = (DOUBLE_3D *) ug_malloc (&ierr, ((*nnbg)+1) * sizeof (DOUBLE_3D));

  if (ierr)
  {
    ug_fclose (Data_File);
    ug_error_message ("*** ERROR 100332 : unable to allocate required memory ***");
    return (100332);
  }

  ir = ir + ug_fread ((*ielielbg)[1], sizeof (INT_4D), *nelbg, Data_File);
  ir = ir - (*nelbg);
  ir = ir + ug_fread ((*inielbg)[1], sizeof (INT_4D), *nelbg, Data_File);
  ir = ir - (*nelbg);
  ir = ir + ug_fread (&((*dfbg)[1]), sizeof (DOUBLE_1D), *nnbg, Data_File);
  ir = ir - (*nnbg);
  if (*mmetbg)
  {
    ir = ir + ug_fread ((*metbg)[1], sizeof (DOUBLE_6D), *nnbg, Data_File);
    ir = ir - (*nnbg);
  }
  ir = ir + ug_fread ((*xbg)[1], sizeof (DOUBLE_3D), *nnbg, Data_File);
  ir = ir - (*nnbg);

  ug_fclose (Data_File);

  if (ir != 0)
  {
    ug_error_message ("*** ERROR 364 : error reading BGDATA file ***");
    return (364);
  }

  if (mmsg >= 1)
  {
    snprintf (Text, sizeof(Text), "UG3      : BG Elements       =%10i", (int) *nelbg);
    ug_message (Text);
    snprintf (Text, sizeof(Text), "UG3      : BG Nodes          =%10i", (int) *nnbg);
    ug_message (Text);
    ug_message ("");

    ug_cpu_message ("UG3      :");
  }

  return (0);
}
